from checkout_sdk.common import ApiResponse
from checkout_sdk.common import Charge


class PaymentHistory(ApiResponse):
    def __init__(self, api_response):
        super().__init__(api_response)
        self._charges = [Charge(charge_data)
                         for charge_data in api_response.body['charges']]

    @property
    def charges(self):
        return self._charges



import unittest
import os
import tests
import json
import logging

import checkout_sdk as sdk

from checkout_sdk import HttpClient, Config, Utils
from checkout_sdk.payments import PaymentsClient, PaymentResponse
from tests.base import CheckoutSdkTestCase
from enum import Enum


class PaymentsClientTests(CheckoutSdkTestCase):
    def setUp(self):
        super().setUp()
        self.http_client = HttpClient(Config())
        self.client = PaymentsClient(self.http_client)

    def tearDown(self):
        super().tearDown()
        self.http_client.close_session()

    def test_bad_payment_response_init(self):
        with self.assertRaises(TypeError):
            PaymentResponse(False)

    def test_payments_client_full_card_auth_request(self):
        payment = self.auth_card()

        self.assertEqual(payment.http_response.status, 200)
        self.assertIsNotNone(payment.created)

        # test payment
        self.assertIsNotNone(payment.id)
        self.assertTrue(payment.approved)
        self.assertIsNotNone(payment.auth_code)
        self.assertEqual(payment.value, 100)
        self.assertEqual(payment.currency, 'USD')
        self.assertEqual(payment.track_id, 'ORDER-001-002')
        self.assertFalse(payment.requires_redirect)

        # test card
        self.assertIsNotNone(payment.card.id)
        self.assertEqual(int(payment.card.expiryMonth), 6)
        self.assertEqual(int(payment.card.expiryYear), 2025)
        self.assertEqual(payment.card.last4, '4242')
        self.assertEqual(payment.card.name, 'Joe Smith')

        # test customer
        self.assertIsNotNone(payment.customer.id)
        self.assertEqual(payment.customer.email, 'joesmith@gmail.com')

        # test other content from the http body
        body = payment.http_response.body

        self.assertEqual(body['card']['billingDetails']['city'], 'London')
        self.assertEqual(body['transactionIndicator'],
                         sdk.PaymentType.Recurring.value)  # pylint: disable = no-member
        self.assertEqual(body['udf1'], 'udf1')
        # below is also a test of snake >> camel casing.
        self.assertEqual(body['customerIp'], '8.8.8.8')
        self.assertEqual(body['products'][0]['price'], 2000)

    def test_payments_client_3d_full_card_auth_request(self):
        payment = self.auth_card(threeds=True)

        self.assertTrue(payment.requires_redirect)
        self.assertTrue(payment.charge_mode ==
                        sdk.ChargeMode.ThreeDS.value)  # pylint: disable = no-member

    def test_payments_client_3d_full_card_auth_request_with_downgrade(self):
        # value 5000 will trigger 20153 (https://docs.checkout.com/docs/testing#section-response-codes)
        payment = self.auth_card(value=5000, threeds=True, attempt_n3d=True)

        self.assertTrue(payment.requires_redirect)
        self.assertTrue(payment.downgraded)

    def test_payments_client_get_request(self):
        payment = self.auth_card()
        # get the previous auth request
        response = self.client.get(payment.id)

        self.assertEqual(response.http_response.status, 200)
        self.assertEqual(payment.id, response.id)
        # TODO: improve test to compare all GET value with previous Auth response values

    def test_payments_history_response(self):
        payment = self.auth_card()
        # history on the previous auth request
        history = self.client.history(payment.id)
        self.assertEqual(history.http_response.status, 200)
        self.assertTrue(len(history.charges) == 1)

        self.assertIsNotNone(history.charges[0].id)
        self.assertEqual(history.charges[0].value, 100)
        self.assertEqual(history.charges[0].currency, 'USD')
        self.assertIsNotNone(history.charges[0].created)
        self.assertIsNotNone(history.charges[0].track_id)
        self.assertIsNotNone(history.charges[0].email)
        self.assertIsNotNone(history.charges[0].status)
        self.assertIsNotNone(history.charges[0].response_code)

        # history after capture
        capture = self.client.capture(payment.id)
        self.assertEqual(capture.http_response.status, 200)

        history = self.client.history(payment.id)
        self.assertEqual(history.http_response.status, 200)
        self.assertTrue(len(history.charges) == 2)

    def test_payments_client_capture_full_amount_request(self):
        payment = self.auth_card(value=150)
        # capture the previous auth request
        response = self.client.capture(payment.id,
                                       track_id='ORDER-001-002-CAPTURE')

        self.assertEqual(response.http_response.status, 200)
        # test payment
        self.assertIsNotNone(response.id)
        self.assertIsNotNone(response.original_id)
        self.assertEqual(payment.id, response.original_id)
        self.assertEqual(response.track_id, 'ORDER-001-002-CAPTURE')
        self.assertEqual(response.value, 150)
        self.assertTrue(response.approved)

    def test_payments_client_capture_partial_amount_request(self):
        payment = self.auth_card(value=150)
        # capture the previous auth request
        response = self.client.capture(payment.id, value=100,
                                       track_id='ORDER-001-002-CAPTURE')

        self.assertEqual(response.http_response.status, 200)
        # test payment
        self.assertIsNotNone(response.id)
        self.assertIsNotNone(response.original_id)
        self.assertEqual(response.track_id, 'ORDER-001-002-CAPTURE')
        self.assertEqual(response.value, 100)
        self.assertTrue(response.approved)

    def test_payments_client_void_request(self):
        payment = self.auth_card()
        # void the previous auth request
        response = self.client.void(payment.id,
                                    track_id='ORDER-001-002-VOID')

        self.assertEqual(response.http_response.status, 200)
        # test payment
        self.assertIsNotNone(response.id)
        self.assertIsNotNone(response.original_id)
        self.assertEqual(response.track_id, 'ORDER-001-002-VOID')
        self.assertTrue(response.approved)

    def test_payments_client_refund_full_amount_request(self):
        payment = self.auth_card(value=150)
        # capture the previous auth request
        capture = self.client.capture(payment.id,
                                      track_id='ORDER-001-002-CAPTURE')
        self.assertEqual(payment.id, capture.original_id)
        # refund the capture
        response = self.client.refund(capture.id,
                                      track_id='ORDER-001-002-REFUND')
        self.assertEqual(response.http_response.status, 200)
        # test payment
        self.assertIsNotNone(response.id)
        self.assertIsNotNone(response.original_id)
        self.assertEqual(capture.id, response.original_id)
        self.assertEqual(response.track_id, 'ORDER-001-002-REFUND')
        self.assertEqual(response.value, 150)
        self.assertTrue(response.approved)

    def test_payments_client_multiple_partial_refunds_request(self):
        payment = self.auth_card(value=150)
        # capture the previous auth request
        capture = self.client.capture(payment.id,
                                      track_id='ORDER-001-002-CAPTURE')
        self.assertEqual(payment.id, capture.original_id)
        # partial refund the capture
        response1 = self.client.refund(capture.id, value=50,
                                       track_id='ORDER-001-002-REFUND-1')
        self.assertTrue(response1.approved)
        self.assertEqual(response1.value, 50)

        response2 = self.client.refund(capture.id, value=80,
                                       track_id='ORDER-001-002-REFUND-2')
        self.assertTrue(response2.approved)
        self.assertEqual(response2.value, 80)

    def auth_card(self, value=None, threeds=False, attempt_n3d=False):
        payment = self.client.request(
            card={
                'number': '4242424242424242',
                'expiryMonth': 6,
                'expiry_year': 2025,  # testing that snake_case is automatically converted
                'cvv': '100',
                'name': 'Joe Smith',
                'billingDetails': {
                    'addressLine1': '1 London Street',
                    'postcode': 'W1',
                    'country': 'GB',
                    'city': 'London',
                    'state': 'Central London',
                    'phone': {
                        'countryCode': '44',
                        'number': '203 123 1234'
                    }
                }
            },
            value=value or 100,  # cents
            currency=sdk.Currency.USD,  # or 'usd'
            auto_capture=False,
            payment_type=sdk.PaymentType.Recurring,
            charge_mode=2 if threeds else 1,
            attempt_n3d=attempt_n3d,
            track_id='ORDER-001-002',
            customer='joesmith@gmail.com',
            udf1='udf1',
            customer_ip='8.8.8.8',
            products=[{
                "description": "Blue Medium",
                "name": "T-Shirt",
                "price": 2000,
                "quantity": 1,
                "shippingCost": 50,
                "sku": "tee123"
            }]
        )

        return payment



import unittest
import os
import tests

from unittest import mock
from tests.base import CheckoutSdkTestCase
from checkout_sdk import Config


class ConfigTests(CheckoutSdkTestCase):
    def setUp(self):
        super().setUp()

    def test_os_env_secret_key_with_sandbox_as_default(self):
        config = Config()
        self.assertEqual(config.secret_key, os.environ['CKO_SECRET_KEY'])
        self.assertRegex(config.api_base_url, r'sandbox')

    def test_os_env_in_production(self):
        with mock.patch.dict('os.environ', {'CKO_SANDBOX': 'False'}):
            config = Config()
            self.assertEqual(config.secret_key, os.environ['CKO_SECRET_KEY'])
            self.assertNotRegex(config.api_base_url, r'sandbox')

    def test_secret_key_in_constructor_overrides_os_env(self):
        self.assertRaises(Exception, Config, secret_key='sk_invalid')



import unittest
import os
import tests
import json

import checkout_sdk as sdk

from checkout_sdk import Config, HttpClient, HttpMethod, errors, constants
from tests.base import CheckoutSdkTestCase
from unittest.mock import MagicMock, Mock, patch
from urllib.parse import urljoin
from requests import Session


class HttpClientTests(CheckoutSdkTestCase):
    RESPONSE_BODY = json.dumps({'key': 'value'})
    PATH = 'test'
    TIMEOUT = constants.DEFAULT_TIMEOUT

    def setUp(self):
        super().setUp()

        self.mock_requests_patcher = patch('checkout_sdk.http_client.requests')
        self.mock_requests = self.mock_requests_patcher.start()

        self.session = MagicMock(spec=Session)
        self.mock_requests.Session = MagicMock(return_value=self.session)

    def tearDown(self):
        super().tearDown()
        self.mock_requests_patcher.stop()

    def set_mock_response(self, name, headers, body, status_code=200):
        response = Mock(name=name, status_code=status_code, headers=headers)
        response.json.return_value = body
        self.session.request = MagicMock(return_value=response)

    def test_http_client_get(self):
        config = Config()
        http_client = HttpClient(config)
        self.set_mock_response(
            'client_get', http_client.headers, self.RESPONSE_BODY)
        response = http_client.send(self.PATH, HttpMethod.GET)

        self.assertEqual(
            str(response), '{0.status} {0.elapsed}'.format(response))
        self.assertEqual(response.body, self.RESPONSE_BODY)
        self.assert_http_call_params(
            path=urljoin(config.api_base_url, self.PATH), method='GET', request=None, headers=response.headers)

    def assert_http_call_params(self, path, method, headers, request):
        self.session.request.assert_called_with(
            url=path,
            method=method,
            headers=headers,
            json=request,
            timeout=self.TIMEOUT/1000
        )



import unittest
import os
import tests
import json

import checkout_sdk as sdk

from checkout_sdk import constants
from checkout_sdk import errors
from tests.base import CheckoutSdkTestCase


class ErrorsTests(CheckoutSdkTestCase):

    def test_init_api_error(self):
        api_error = errors.ApiError(
            message='api error',
            event_id='event_id',
            http_status=500,
            error_code=500,
            elapsed=100)
        self.assertEqual(api_error.message, 'api error')
        self.assertEqual(api_error.event_id, 'event_id')
        self.assertEqual(api_error.http_status, 500)
        self.assertEqual(api_error.error_code, 500)
        self.assertEqual(api_error.elapsed, 100)

        self.assertEqual(
            str(api_error), '{0.event_id} - {0.message}'.format(api_error))

    def test_init_bad_request_error(self):
        bad_request_error = errors.BadRequestError(
            message='bad request',
            error_code=constants.VALIDATION_ERROR_CODE,
            json={
                "errorCode": "70000",
                "message": "Validation error",
                "errorMessageCodes": [
                    "001",
                    "002",
                    "003"
                ],
                "errors": [
                    "Validation error 1",
                    "Validation error 2"
                ]
            }
        )
        self.assertEqual(bad_request_error.message, 'bad request')
        self.assertTrue(bad_request_error.validation_error)
        self.assertTrue(
            bad_request_error.errors['001'] == 'Validation error 1')
        self.assertTrue(
            bad_request_error.errors['002'] == 'Validation error 2')
        self.assertTrue(bad_request_error.errors['003'] == '')

    def test_init_authentication_error(self):
        error = errors.AuthenticationError()
        self.assertIsNotNone(error.message)

    def test_init_resource_not_found_error(self):
        error = errors.ResourceNotFoundError()
        self.assertIsNotNone(error.message)

    def test_timeout_error(self):
        error = errors.Timeout()
        self.assertIsNotNone(error.message)

    def test_too_many_requests_error(self):
        error = errors.TooManyRequestsError()
        self.assertIsNotNone(error.message)



from checkout_sdk.payments import PaymentResponse


class PaymentActionResponse(PaymentResponse):
    @property
    def created(self):
        return self._response.body['created']

    @property
    def track_id(self):
        return self._response.body['trackId']

    @property
    def currency(self):
        return self._response.body['currency']

    @property
    def value(self):
        return self._response.body['value']

    @property
    def response_code(self):
        return self._response.body['responseCode']

    @property
    def approved(self):
        return str(self.response_code).startswith('1')



from checkout_sdk.payments import PaymentActionResponse


class RefundResponse(PaymentActionResponse):
    def __init__(self, api_response):
        super().__init__(api_response)

    @property
    def original_id(self):
        return self._response.body['originalId']



import unittest
import os
import tests
import json

import checkout_sdk as sdk

from checkout_sdk import Utils, errors
from tests.base import CheckoutSdkTestCase
from checkout_sdk.common import HttpResponse


class UtilsTests(CheckoutSdkTestCase):
    VALID_ID = 'id'
    INVALID_ID = False

    def test_validate_id(self):
        try:
            Utils.validate_id(self.VALID_ID)
        except Exception:
            self.fail(
                'Utils.validate_payment_id raised an exception unexpectedly.')

    def test_validate_id_with_missing_id(self):
        with self.assertRaises(ValueError):
            Utils.validate_id(None)

    def test_validate_id_with_wrong_id_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_id(self.INVALID_ID)

    def test_validate_payment_source_card(self):
        try:
            Utils.validate_payment_source(self.VALID_ID)
        except Exception:
            self.fail(
                'Utils.validate_payment_source raised an exception unexpectedly for a valid card source.')

    def test_validate_payment_source_token(self):
        try:
            Utils.validate_payment_source(None, self.VALID_ID)
        except Exception:
            self.fail(
                'Utils.validate_payment_source raised an exception unexpectedly for a valid token.')

    def test_validate_payment_source_with_missing_source(self):
        with self.assertRaises(ValueError):
            Utils.validate_payment_source(None, None)

    def test_validate_payment_source_card_id_fails(self):
        with self.assertRaises(TypeError):
            Utils.validate_payment_source(self.INVALID_ID)

    def test_validate_payment_source_token_fails(self):
        with self.assertRaises(TypeError):
            Utils.validate_payment_source(None, self.INVALID_ID)

    def test_validate_customer_with_missing_id(self):
        with self.assertRaises(ValueError):
            Utils.validate_customer(None)

    def test_validate_customer_with_wrong_id_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_customer(self.INVALID_ID)

    def test_validate_transaction(self):
        try:
            Utils.validate_transaction(
                100, sdk.Currency.USD, sdk.PaymentType.Recurring, sdk.ChargeMode.NonThreeD)
        except Exception:
            self.fail(
                'Utils.validate_transaction raised an exception unexpectedly when using enums.')

    def test_validate_transaction_without_enums(self):
        try:
            Utils.validate_transaction(100, 'eur', 2, 1)
        except Exception:
            self.fail(
                'Utils.validate_transaction raised an exception unexpectedly when not using enums')

    def test_validate_transaction_fails_with_missing_value(self):
        with self.assertRaises(ValueError):
            Utils.validate_transaction(None)

    def test_validate_transaction_fails_with_negative_value(self):
        with self.assertRaises(ValueError):
            Utils.validate_transaction(-5)

    def test_validate_transaction_fails_with_wrong_value_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_transaction('value')

    def test_validate_transaction_fails_with_bad_currency(self):
        with self.assertRaises(ValueError):
            Utils.validate_transaction(100, 'xxx')

    def test_validate_transaction_fails_with_wrong_currency_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_transaction(100, False)

    def test_validate_transaction_fails_with_bad_payment_type(self):
        with self.assertRaises(ValueError):
            Utils.validate_transaction(100, 'usd', 5)

    def test_validate_transaction_fails_with_wrong_payment_type_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_transaction(100, 'usd', False)

    def test_validate_transaction_fails_with_bad_charge_mode(self):
        with self.assertRaises(ValueError):
            Utils.validate_transaction(100, 'usd', 2, 10)

    def test_validate_transaction_fails_with_wrong_charge_mode_type(self):
        with self.assertRaises(TypeError):
            Utils.validate_transaction(100, 'usd', 2, False)

    def test_verify_redirect_flow(self):
        http_response = HttpResponse(200, None, {
            'redirectUrl': 'http',
            'id': 'pay_tok_1'
        }, 0)
        self.assertTrue(Utils.verify_redirect_flow(http_response))



import re
import os

from checkout_sdk import constants, logger

SECRET_KEY_REGEX = re.compile(
    r'^sk_(test_)?(\w{8})-(\w{4})-(\w{4})-(\w{4})-(\w{12})$', re.IGNORECASE)
HTTP_PROTOCOL_REGEX = re.compile(r'^http(s)?', re.IGNORECASE)

api_base_urls = {
    'sandbox': 'https://sandbox.checkout.com/api2/v2/',
    'production': 'https://api2.checkout.com/v2/'
}

# using lambda functions here so that env variables are read at the time of access
env_settings = {
    'secret_key': lambda: os.environ.get('CKO_SECRET_KEY'),
    'sandbox': lambda: os.environ.get('CKO_SANDBOX', 'true').lower() in ['true', '1']
}


class Config:
    def __init__(self, secret_key=None, sandbox=None, timeout=constants.DEFAULT_TIMEOUT, api_base_url=None):
        self.secret_key = \
            secret_key if secret_key else env_settings['secret_key']()
        self.timeout = timeout
        # use custom api_base_url if local dev, integration testing, etc
        # else `sandbox` determines the URL
        if api_base_url:
            self.api_base_url = api_base_url
        else:
            sandbox = \
                sandbox if sandbox is not None else env_settings['sandbox']()
            self.api_base_url = api_base_urls['sandbox'] if sandbox == True else api_base_urls['production']

        logger.info('{} - sk{}{}'.format(self.api_base_url, '*' * 6,
                                         self.secret_key[-6:]))

    @property
    def secret_key(self):
        return self._secret_key

    @secret_key.setter
    def secret_key(self, value):
        if SECRET_KEY_REGEX.match(value or ''):
            self._secret_key = value
        else:
            raise ValueError('Invalid secret key.')

    @property
    def api_base_url(self):
        return self._api_base_url

    @api_base_url.setter
    def api_base_url(self, value):
        # TODO: Test for valid URL and http[s]
        self._api_base_url = value

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        try:
            self._timeout = int(value)
        except ValueError:
            raise ValueError(
                'Invalid timeout. Default is {} milliseconds'.format(constants.DEFAULT_TIMEOUT))



import checkout_sdk as sdk

from checkout_sdk import ApiClient, Utils, HttpMethod
from checkout_sdk.tokens import PaymentTokenResponse


class TokensClient(ApiClient):
    def request_payment_token(self,
                              # Transaction
                              value=0, currency=sdk.default_currency,
                              track_id=None,
                              # Auto Capture
                              auto_capture=sdk.default_auto_capture,
                              auto_capture_delay=sdk.default_auto_capture_delay,
                              # Urls
                              success_url=None,
                              fail_url=None,
                              **kwargs):

        self._log_info(
            'Payment token {}{} - {}'.format(value, currency, track_id if track_id else '<no track id>'))

        Utils.validate_transaction(value=value, currency=currency)

        request = {
            'value': value,
            'currency': currency if not isinstance(currency, sdk.Currency) else currency.value,
            'trackId': track_id,
            'chargeMode': sdk.ChargeMode.APM.value,  # pylint: disable = no-member
            'autoCapture': 'Y' if auto_capture else 'N',
            'autoCapTime': auto_capture_delay,
            'successUrl': success_url,
            'failUrl': fail_url
        }

        # add remaining properties
        request.update(kwargs)

        http_response = self._send_http_request('tokens/payment',
                                                HttpMethod.POST, request)

        return PaymentTokenResponse(http_response)



from checkout_sdk.payments.payment_response import PaymentResponse
from checkout_sdk.payments.payment_action_response import PaymentActionResponse
from checkout_sdk.payments.payment_history import PaymentHistory
from checkout_sdk.payments.payment_processed import PaymentProcessed
from checkout_sdk.payments.threeds_response import ThreeDSResponse
from checkout_sdk.payments.capture_response import CaptureResponse
from checkout_sdk.payments.void_response import VoidResponse
from checkout_sdk.payments.refund_response import RefundResponse
from checkout_sdk.payments.payments_client import PaymentsClient



import re
import pprint

from checkout_sdk import HttpMethod, logger

SNAKE_CASE_REGEX = re.compile(r'_([a-z])')


def _snake_to_camel_case(name):
    return SNAKE_CASE_REGEX.sub(lambda x: x.group(1).upper(), name)


class ApiClient:
    def __init__(self, http_client):
        self._http_client = http_client

    def _send_http_request(self, url, method, request=None):
        if request:
            request = self._convert_json_case(request)

        response = self._http_client.send(url, method, request)
        self._log_info(response)        # http status, elapsed
        self._log_debug('HTTP response:\n'+pprint.pformat(response.body))
        return response

    def _convert_json_case(self, json):
        output = {}
        for k, v in json.items():
            output[_snake_to_camel_case(k)] = self._convert_json_case(
                v) if isinstance(v, dict) else v
        return output

    def _log_info(self, message, **kwargs):
        logger.info(message, **kwargs)

    def _log_debug(self, message, **kwargs):
        logger.debug(message, **kwargs)



import logging
import os
import sys

name = "checkout_sdk"

# defaults

from checkout_sdk.enums import Currency, ChargeMode, HttpMethod, PaymentType

default_currency = None
default_auto_capture = True
default_auto_capture_delay = 0  # valid: 0 - 168 (hours)
default_payment_type = PaymentType.Regular

# logging
logger = logging.getLogger('cko')
formatter = logging.Formatter('[%(levelname)s] %(message)s')
handler = logging.StreamHandler(stream=sys.stdout)
handler.setFormatter(formatter)
logger.addHandler(handler)

logging_levels = {
    'debug': logging.DEBUG,
    'info': logging.INFO
}
env_logging_level = str(os.environ.get('CKO_LOGGING', None)).lower()
if env_logging_level in logging_levels.keys():
    logger.setLevel(logging_levels[env_logging_level])

# class imports (order matters!)

from checkout_sdk import constants
from checkout_sdk.config import Config
from checkout_sdk.utils import Utils

from checkout_sdk.http_client import HttpClient
from checkout_sdk.api_client import ApiClient
from checkout_sdk.checkout_api import CheckoutApi


def get_api(secret_key=None, sandbox=None, timeout=constants.DEFAULT_TIMEOUT, api_base_url=None):
    return CheckoutApi(**locals())



from enum import Enum


class ExtendedEnum(Enum):
    @classmethod
    def has_value(cls, value):
        return value is not None and any(str(value).lower() == str(item.value).lower() for item in cls)

    def __str__(self):
        return '{0.value}'.format(self)


class PaymentType(ExtendedEnum):
    Regular = 1
    Recurring = 2
    MOTO = 3


class ChargeMode(ExtendedEnum):
    NonThreeD = 1
    ThreeDS = 2
    APM = 3


class HttpMethod(ExtendedEnum):
    GET = 'GET'
    POST = 'POST'


class Currency(ExtendedEnum):
    ALL = "ALL"
    STN = "STN"
    EEK = "EEK"
    BHD = "BHD"
    SCR = "SCR"
    DJF = "DJF"
    EGP = "EGP"
    MDL = "MDL"
    MZN = "MZN"
    BND = "BND"
    ZMK = "ZMK"
    SHP = "SHP"
    LBP = "LBP"
    AWG = "AWG"
    JMD = "JMD"
    KES = "KES"
    BYN = "BYN"
    KHR = "KHR"
    LAK = "LAK"
    MVR = "MVR"
    AOA = "AOA"
    TJS = "TJS"
    SVC = "SVC"
    GNF = "GNF"
    BRL = "BRL"
    MOP = "MOP"
    BOB = "BOB"
    CDF = "CDF"
    NAD = "NAD"
    LYD = "LYD"
    VUV = "VUV"
    QAR = "QAR"
    CLP = "CLP"
    HRK = "HRK"
    ISK = "ISK"
    FKP = "FKP"
    XCD = "XCD"
    NOK = "NOK"
    CUP = "CUP"
    VND = "VND"
    PEN = "PEN"
    KMF = "KMF"
    LVL = "LVL"
    MMK = "MMK"
    TRY = "TRY"
    VEF = "VEF"
    AUD = "AUD"
    TWD = "TWD"
    PKR = "PKR"
    SLL = "SLL"
    BGN = "BGN"
    LRD = "LRD"
    LKR = "LKR"
    XAF = "XAF"
    JOD = "JOD"
    ANG = "ANG"
    BSD = "BSD"
    CAD = "CAD"
    GIP = "GIP"
    MNT = "MNT"
    LTL = "LTL"
    BBD = "BBD"
    CLF = "CLF"
    BWP = "BWP"
    COP = "COP"
    PHP = "PHP"
    HUF = "HUF"
    FJD = "FJD"
    MWK = "MWK"
    THB = "THB"
    XPF = "XPF"
    RSD = "RSD"
    SAR = "SAR"
    UYU = "UYU"
    BZD = "BZD"
    SYP = "SYP"
    GMD = "GMD"
    SZL = "SZL"
    SBD = "SBD"
    ETB = "ETB"
    CHF = "CHF"
    MXN = "MXN"
    ARS = "ARS"
    GTQ = "GTQ"
    GHS = "GHS"
    NIO = "NIO"
    JPY = "JPY"
    BDT = "BDT"
    UZS = "UZS"
    SOS = "SOS"
    BTN = "BTN"
    NZD = "NZD"
    TZS = "TZS"
    IQD = "IQD"
    MGA = "MGA"
    DZD = "DZD"
    GYD = "GYD"
    USD = "USD"
    KWD = "KWD"
    CNY = "CNY"
    PYG = "PYG"
    SGD = "SGD"
    KZT = "KZT"
    PGK = "PGK"
    AMD = "AMD"
    GBP = "GBP"
    AFN = "AFN"
    CRC = "CRC"
    XOF = "XOF"
    YER = "YER"
    MRU = "MRU"
    DKK = "DKK"
    TOP = "TOP"
    INR = "INR"
    SDG = "SDG"
    DOP = "DOP"
    ZWL = "ZWL"
    UGX = "UGX"
    SEK = "SEK"
    LSL = "LSL"
    MYR = "MYR"
    TMT = "TMT"
    OMR = "OMR"
    BMD = "BMD"
    KRW = "KRW"
    HKD = "HKD"
    KGS = "KGS"
    BAM = "BAM"
    NGN = "NGN"
    ILS = "ILS"
    MUR = "MUR"
    RON = "RON"
    TND = "TND"
    AED = "AED"
    PAB = "PAB"
    NPR = "NPR"
    TTD = "TTD"
    RWF = "RWF"
    HTG = "HTG"
    IDR = "IDR"
    EUR = "EUR"
    KYD = "KYD"
    IRR = "IRR"
    KPW = "KPW"
    MKD = "MKD"
    SRD = "SRD"
    HNL = "HNL"
    AZN = "AZN"
    ERN = "ERN"
    CZK = "CZK"
    CVE = "CVE"
    BIF = "BIF"
    MAD = "MAD"
    RUB = "RUB"
    UAH = "UAH"
    WST = "WST"
    PLN = "PLN"
    ZAR = "ZAR"
    GEL = "GEL"
    ZMW = "ZMW"



class Charge:
    def __init__(self, charge):
        self._charge = charge

    @property
    def id(self):
        return self._charge['id']

    @property
    def charge_mode(self):
        return self._charge['chargeMode']

    @property
    def created(self):
        return self._charge['created']

    @property
    def email(self):
        return self._charge['email']

    @property
    def live_mode(self):
        return self._charge['liveMode']

    @property
    def status(self):
        return self._charge['status']

    @property
    def track_id(self):
        return self._charge['trackId']

    @property
    def value(self):
        return self._charge['value']

    @property
    def currency(self):
        return self._charge['currency']

    @property
    def response_code(self):
        return self._charge['responseCode']



from checkout_sdk import Config, HttpClient
from checkout_sdk.payments import PaymentsClient
from checkout_sdk.tokens import TokensClient


class CheckoutApi:
    def __init__(self, **kwargs):
        http_client = HttpClient(Config(**kwargs))
        self.payments = PaymentsClient(http_client)
        self.tokens = TokensClient(http_client)



import unittest
import os
import tests
import json

import checkout_sdk as sdk

from checkout_sdk import HttpClient, Config, Utils
from checkout_sdk.tokens import TokensClient, PaymentTokenResponse
from tests.base import CheckoutSdkTestCase
from enum import Enum


class TokensClientTests(CheckoutSdkTestCase):
    def setUp(self):
        super().setUp()
        self.http_client = HttpClient(Config())
        self.client = TokensClient(self.http_client)

    def tearDown(self):
        super().tearDown()
        self.http_client.close_session()

    def test_bad_payment_token_response_init(self):
        with self.assertRaises(TypeError):
            PaymentTokenResponse(False)

    def test_tokens_client_payment_token_request(self):
        token = self.client.request_payment_token(
            value=100,                  # cents
            currency=sdk.Currency.USD   # or 'usd'
        )

        self.assertEqual(token.http_response.status, 200)

        # test payment
        self.assertIsNotNone(token.id)



import checkout_sdk as sdk

from checkout_sdk.payments import PaymentResponse


class ThreeDSResponse(PaymentResponse):
    def __init__(self, api_response):
        super().__init__(api_response)

    @property
    def charge_mode(self):
        return self._response.body['chargeMode']

    @property
    def track_id(self):
        return self._response.body['trackId']

    @property
    def response_code(self):
        return self._response.body['responseCode']

    @property
    def enrolled(self):
        return self._response.body['enrolled'] == 'Y'

    @property
    def redirect_url(self):
        return self._response.body['redirectUrl']

    @property
    def downgraded(self):
        return self.charge_mode == sdk.ChargeMode.NonThreeD.value  # pylint: disable = no-member



from checkout_sdk.common import ApiResponse

from checkout_sdk import Utils


class PaymentResponse(ApiResponse):
    @property
    def id(self):
        return self._response.body['id']

    @property
    def requires_redirect(self):
        return Utils.verify_redirect_flow(self._response)



import re
import datetime
import checkout_sdk
import pprint

from checkout_sdk import errors, constants
from checkout_sdk.enums import Currency, PaymentType, ChargeMode

EMAIL_REGEX = re.compile(r'^.+@.+$', re.IGNORECASE)
PAYMENT_TOKEN_REGEX = re.compile(r'^pay_tok_', re.IGNORECASE)


class Utils:
    @classmethod
    def validate_id(cls, id):
        """Validates the payment id."""
        if id is None:
            raise ValueError('Invalid Payment/Charge Id.')
        if type(id) is not str:
            raise TypeError('Id should be of string type.')

    @classmethod
    def validate_payment_source(cls, card=None, token=None):
        """Validates card and token payment sources."""
        if card is None and token is None:
            raise ValueError(
                'Payment source missing. Please specify a valid card or token.')
        if card is not None and not (isinstance(card, dict) or type(card) is str):
            raise TypeError(
                'Invalid card source. Please provide a valid Card Id.')
        if token is not None and type(token) is not str:
            raise TypeError('Invalid token source.')

    @classmethod
    def validate_customer(cls, customer):
        if customer is None:
            raise ValueError(
                'Email or Customer Id is required when requesting a payment.')
        if type(customer) is not str:
            raise TypeError(
                'Customer should be of string type')

    @classmethod
    def is_email(cls, val):
        return EMAIL_REGEX.match(val) is not None

    @classmethod
    def validate_transaction(cls, value, currency=None, payment_type=None, charge_mode=None):
        if value is None or (type(value) is int and value < 0):
            raise ValueError('Value must be greater or equal to zero.')
        if type(value) is not int:
            raise TypeError('Value must be an integer.')
        if currency is not None and type(currency) is str and not Currency.has_value(currency):
            raise ValueError('Invalid currency.')
        if currency is not None and not (isinstance(currency, Currency) or type(currency) is str):
            raise TypeError(
                'Currency should be of the correct enum type or a string.')
        if payment_type is not None and type(payment_type) is int and not PaymentType.has_value(payment_type):
            raise ValueError('Invalid payment type.')
        if payment_type is not None and not (isinstance(payment_type, PaymentType) or type(payment_type) is int):
            raise TypeError(
                'Payment type should be of the correct enum type or an integer.')
        if charge_mode is not None and type(charge_mode) is int and not ChargeMode.has_value(charge_mode):
            raise ValueError('Invalid charge mode.')
        if charge_mode is not None and not (isinstance(charge_mode, ChargeMode) or type(charge_mode) is int):
            raise TypeError(
                'Charge mode should be of the correct enum type or an integer.')

    @classmethod
    def mask_pan(cls, pan):
        left, right = (6, 4)
        return pan[0:left].ljust(len(pan)-right, '*')+pan[-right:]

    @classmethod
    def verify_redirect_flow(cls, http_response):
        # this is how a 3D response is detected currently
        return http_response.body.get('redirectUrl') is not None \
            and PAYMENT_TOKEN_REGEX.match(http_response.body.get('id')) is not None



from checkout_sdk import constants


class CheckoutSdkError(Exception):
    def __init__(self,
                 event_id=None,
                 http_status=None,
                 error_code=None,
                 message=None,
                 elapsed=0,
                 json=None):
        super().__init__(message)
        self.event_id = event_id
        self.http_status = http_status
        self.error_code = error_code
        self.message = message
        self.elapsed = elapsed
        self.json = json

    def __str__(self):
        return '{} - {}'.format(self.event_id, self.message)


class AuthenticationError(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop(
            'message', 'Authentication error. Secret key might be missing or expired.')
        super().__init__(message=message, **kwargs)


class BadRequestError(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop(
            'message', 'Bad request. Check for possible validation error.')
        super().__init__(message=message, **kwargs)

        self._errors = {}
        if self.json is not None:
            error_message_codes = self.json.get('errorMessageCodes', [])
            errors = self.json.get('errors', [])
            index = 0
            for code in error_message_codes:
                self._errors[code] = errors[index] if index < len(
                    errors) else ''
                index = index + 1

    @property
    def validation_error(self):
        return self.error_code == constants.VALIDATION_ERROR_CODE

    @property
    def errors(self):
        return self._errors


class ResourceNotFoundError(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop(
            'message', 'Resource not found. Please check the identifier and retry.')
        super().__init__(message=message, **kwargs)


class Timeout(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop(
            'message', 'The request timed out. Considering adjust the "timeout" setting.')
        super().__init__(message=message, **kwargs)


class TooManyRequestsError(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop(
            'message', 'Requests blocked due to API request throttling.')
        super().__init__(message=message, **kwargs)


class ApiError(CheckoutSdkError):
    def __init__(self, **kwargs):
        message = kwargs.pop('message', 'General API error.')
        super().__init__(message=message, **kwargs)



from checkout_sdk.common import HttpResponse


class ApiResponse:
    def __init__(self, response):
        if not isinstance(response, HttpResponse):
            raise TypeError(
                'response must be a valid instance of HttpResponse')
        self._response = response

    @property
    def http_response(self):
        """Http response with status, headers, JSON body and elapsed time (ms)."""
        return self._response



import unittest
import os
import tests

from checkout_sdk import logger


class CheckoutSdkTestCase(unittest.TestCase):
    def setUp(self):
        super().setUp()
        print('\n')

    def tearDown(self):
        super().tearDown()
        print('\n')



from checkout_sdk.payments import PaymentActionResponse
from checkout_sdk.common import Customer, Card


class PaymentProcessed(PaymentActionResponse):
    def __init__(self, api_response):
        super().__init__(api_response)
        self._card = Card(api_response.body['card'])
        # customer name is not currently returned from an Auth response
        self._customer = Customer(
            id=api_response.body['card']['customerId'], email=api_response.body['email'])

    @property
    def charge_mode(self):
        return self._response.body['chargeMode']

    @property
    def auth_code(self):
        return self._response.body['authCode']

    @property
    def card(self):
        return self._card

    @property
    def customer(self):
        return self._customer



import requests
import time

from checkout_sdk import errors, constants, HttpMethod
from checkout_sdk.common import HttpResponse
from urllib.parse import urljoin

http_headers_default = {
    'user-agent': 'checkout-sdk-python/{}'.format(constants.VERSION)
}


class HttpClient:
    def __init__(self, config):
        self.config = config

        # init Http Session (for pooling)
        self._session = requests.Session()

        # interceptor call is a mirror by default
        self.interceptor = lambda url, headers, request: (
            url, headers, request)

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value):
        self._config = value

    @property
    def headers(self):
        headers = http_headers_default.copy()
        headers['authorization'] = self.config.secret_key
        return headers

    def close_session(self):
        self._session.close()

    def send(self, path, method=HttpMethod.GET, request=None):
        start = time.time()

        # call the interceptor as a hook to override the url, headers and/or request
        url, headers, request = self.interceptor(
            urljoin(self.config.api_base_url, path), self.headers, request)

        try:
            r = self._session.request(
                method=method.value,
                url=url,
                json=request,
                headers=headers,
                timeout=self.config.timeout/1000)
            elapsed = '{0:.2f}'.format((time.time() - start)*1000)

            r.raise_for_status()
            try:
                body = r.json()
            except ValueError:
                body = None

            return HttpResponse(r.status_code, r.headers, body, elapsed)
        except requests.exceptions.HTTPError as e:
            status_code_switch = {
                400: lambda: errors.BadRequestError,
                401: lambda: errors.AuthenticationError,
                404: lambda: errors.ResourceNotFoundError,
                422: lambda: errors.TooManyRequestsError,
                500: lambda: errors.ApiError
            }
            try:
                jsonResponse = e.response.json()
            except ValueError:
                jsonResponse = None

            errorCls = status_code_switch.get(e.response.status_code,
                                              errors.ApiError)()
            raise errorCls(
                event_id=jsonResponse['eventId'],
                http_status=e.response.status_code,
                error_code=jsonResponse['errorCode'],
                message=jsonResponse['message'],
                elapsed=elapsed,
                json=jsonResponse)
        except requests.exceptions.Timeout as e:
            elapsed = time.time() - start
            raise errors.Timeout(elapsed=elapsed)
        except requests.exceptions.RequestException:
            raise errors.ApiError(
                message='Unexpected API connection error - please contact support@checkout.com')



class Card:
    def __init__(self, card):
        self._card = card

    @property
    def id(self):
        return self._card['id']

    @property
    def scheme(self):
        return self._card['paymentMethod']

    @property
    def last4(self):
        return self._card['last4']

    @property
    def expiry_month(self):
        return self._card['expiryMonth']

    @property
    def expiry_year(self):
        return self._card['expiryYear']

    @property
    def name(self):
        return self._card['name']

    """
    ******************************************************************
    BACKWARDS COMPATIBILITY - Please use the snake_case versions above
    ******************************************************************
    """
    @property
    def expiryMonth(self):
        return self._card['expiryMonth']

    @property
    def expiryYear(self):
        return self._card['expiryYear']



import checkout_sdk as sdk

from checkout_sdk import ApiClient, Utils, HttpMethod
from checkout_sdk.payments import (
    PaymentHistory,
    PaymentProcessed,
    ThreeDSResponse,
    CaptureResponse,
    VoidResponse,
    RefundResponse
)


class PaymentsClient(ApiClient):
    def request(self,
                # Source
                card=None, token=None,
                # Transaction
                value=0, currency=sdk.default_currency,
                payment_type=sdk.default_payment_type,
                customer=None, track_id=None,
                # Auto Capture
                auto_capture=sdk.default_auto_capture,
                auto_capture_delay=sdk.default_auto_capture_delay,
                # 3D
                charge_mode=sdk.ChargeMode.NonThreeD,
                attempt_n3d=False,
                **kwargs):

        payment_source = card or token

        # card can be a dictionary and need the JSON case converted, if needed, before being validated
        if isinstance(card, dict):
            card = self._convert_json_case(card)
            # change payment source to a masked PAN
            payment_source = Utils.mask_pan(card['number'])

        self._log_info(
            'Auth {} - {}{} - {}'.format(payment_source, value, currency, track_id if track_id else '<no track id>'))

        Utils.validate_payment_source(card=card, token=token)
        Utils.validate_transaction(
            value=value, currency=currency, payment_type=payment_type, charge_mode=charge_mode)
        Utils.validate_customer(customer=customer)

        request = {
            'value': value,
            'currency': currency if not isinstance(currency, sdk.Currency) else currency.value,
            'trackId': track_id,
            'transactionIndicator': payment_type if not isinstance(payment_type,
                                                                   sdk.PaymentType) else payment_type.value,
            'chargeMode': charge_mode if not isinstance(charge_mode, sdk.ChargeMode) else charge_mode.value,
            'attemptN3D': attempt_n3d,
            'autoCapture': 'Y' if auto_capture else 'N',
            'autoCapTime': auto_capture_delay
        }

        if card:
            if isinstance(card, dict):
                request['card'] = card
            else:
                request['cardId'] = card
        else:
            request['cardToken'] = token

        if Utils.is_email(customer):
            request['email'] = customer
        else:
            request['customerId'] = customer

        # add remaining properties
        request.update(kwargs)

        http_response = self._send_http_request('charges/card' if card is not None else 'charges/token',
                                                HttpMethod.POST, request)

        if Utils.verify_redirect_flow(http_response):
            return ThreeDSResponse(http_response)
        else:
            return PaymentProcessed(http_response)

    def capture(self, id, value=None, track_id=None, **kwargs):
        return CaptureResponse(self._getPaymentActionResponse(id, 'capture', value, track_id, **kwargs))

    def void(self, id, track_id=None, **kwargs):
        return VoidResponse(self._getPaymentActionResponse(id, 'void', None, track_id, **kwargs))

    def refund(self, id, value=None, track_id=None, **kwargs):
        return RefundResponse(self._getPaymentActionResponse(id, 'refund', value, track_id, **kwargs))

    def get(self, id):
        self._log_info('Get {}'.format(id))

        Utils.validate_id(id)

        return PaymentProcessed(self._send_http_request('charges/{}'.format(id), HttpMethod.GET))

    def history(self, id):
        self._log_info('Getting charge history for {}'.format(id))

        Utils.validate_id(id)

        return PaymentHistory(self._send_http_request('charges/{}/history'.format(id), HttpMethod.GET))

    def _getPaymentActionResponse(self, id, action, value, track_id, **kwargs):
        self._log_info('{} - {} - {}'.format(action.capitalize(), id,
                                             track_id if track_id else '<no track id>'))

        Utils.validate_id(id)

        request = {
            'trackId': track_id
        }

        if value is not None:
            Utils.validate_transaction(value=value)
            request['value'] = value

        # add remaining properties
        request.update(kwargs)

        return self._send_http_request(
            'charges/{}/{}'.format(id, action), HttpMethod.POST, request)



import setuptools
import os

with open("README.md", "r") as f:
    long_description = f.read()

constants = {}
with open(os.path.join('checkout_sdk', 'constants.py')) as f:
    exec(f.read(), constants)

setuptools.setup(
    name="checkout_sdk",
    version=constants['VERSION'],
    author="Checkout.com",
    author_email="support@checkout.com",
    description="Checkout.com Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    zip_safe=False,
    url="https://github.com/checkout/checkout-sdk-python",
    license='MIT',
    packages=setuptools.find_packages(),
    install_requires=[
        'requests >= 2.0.0',
    ],
    test_suite='tests',
    classifiers=[
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ]
)



import unittest
import os
import tests
import json

import checkout_sdk as sdk
from checkout_sdk import HttpClient
from checkout_sdk.payments import PaymentsClient
from checkout_sdk.tokens import TokensClient
from tests.base import CheckoutSdkTestCase


class CheckoutApiTests(CheckoutSdkTestCase):

    def test_init_checkout_api(self):
        api = sdk.get_api()

        self.assertTrue(isinstance(api.payments, PaymentsClient))
        self.assertTrue(isinstance(api.tokens, TokensClient))
        self.assertTrue(isinstance(api.payments._http_client, HttpClient))



class Customer:
    def __init__(self, id, email=None, name=None):
        self._id = id
        self._email = email
        self._name = name

    @property
    def id(self):
        return self._id

    @property
    def email(self):
        return self._email

    @property
    def name(self):
        return self._name



from checkout_sdk.common.http_response import HttpResponse
from checkout_sdk.common.customer import Customer
from checkout_sdk.common.card import Card
from checkout_sdk.common.charge import Charge
from checkout_sdk.common.api_response import ApiResponse
